import React from 'react';
import { EditorView } from '@codemirror/view';
import { Compartment } from '@codemirror/state';
import { createEditorView, replaceWholeDoc } from '@ui-schema/kit-codemirror/createEditorView';
export const useCodeMirror = (containerRef, onChange, value = '', extensions, effects, onExternalChange = replaceWholeDoc) => {
  var _effects;

  const lastValueRef = React.useRef(value);
  const onChangeRef = React.useRef(undefined);
  onChangeRef.current = onChange;
  const readOnlyCompartment = React.useRef(new Compartment());
  const editor = React.useMemo(() => {
    return createEditorView(containerRef.current, lastValueRef, [...(extensions || []), readOnlyCompartment.current.of(EditorView.editable.of(Boolean(onChangeRef.current)))], onChangeRef);
  }, [containerRef, lastValueRef, extensions, onChangeRef]);
  React.useLayoutEffect(() => {
    var _containerRef$current;

    if (!editor) {
      return;
    }

    (_containerRef$current = containerRef.current) === null || _containerRef$current === void 0 ? void 0 : _containerRef$current.append(editor.dom);
    return () => {
      editor === null || editor === void 0 ? void 0 : editor.destroy();
    };
  }, [containerRef, editor]);
  effects = ((_effects = effects) === null || _effects === void 0 ? void 0 : _effects.length) === 0 ? undefined : effects;
  React.useLayoutEffect(() => {
    if (effects && effects.length > 0 && !editor) {
      console.error('received effects but editor is not ready', effects);
      return;
    }

    if (!effects || !editor) return;
    effects.forEach(effect => {
      effect(editor);
    });
  }, [effects, editor]);
  React.useEffect(() => {
    if (!editor) {
      return;
    }

    editor.dispatch({
      effects: readOnlyCompartment.current.reconfigure(EditorView.editable.of(Boolean(onChange)))
    });
  }, [editor, onChange]);
  React.useLayoutEffect(() => {
    if (editor && containerRef.current && lastValueRef.current !== value) {
      if (lastValueRef.current === value) {
        return;
      }

      const textA = editor.state.doc.toString();
      const textB = value;
      onExternalChange(editor, textB, textA);
      lastValueRef.current = textB;
    } else {
      lastValueRef.current = value;
    }
  }, [containerRef, value, editor, onExternalChange]);
  return editor;
};