"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useCodeMirror = void 0;

var _react = _interopRequireDefault(require("react"));

var _view = require("@codemirror/view");

var _state = require("@codemirror/state");

var _createEditorView = require("@ui-schema/kit-codemirror/createEditorView");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var useCodeMirror = function useCodeMirror(containerRef, onChange) {
  var _effects;

  var value = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
  var extensions = arguments.length > 3 ? arguments[3] : undefined;
  var effects = arguments.length > 4 ? arguments[4] : undefined;
  var onExternalChange = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : _createEditorView.replaceWholeDoc;

  var lastValueRef = _react["default"].useRef(value);

  var onChangeRef = _react["default"].useRef(undefined);

  onChangeRef.current = onChange;

  var readOnlyCompartment = _react["default"].useRef(new _state.Compartment());

  var editor = _react["default"].useMemo(function () {
    return (0, _createEditorView.createEditorView)(containerRef.current, lastValueRef, [].concat(_toConsumableArray(extensions || []), [readOnlyCompartment.current.of(_view.EditorView.editable.of(Boolean(onChangeRef.current)))]), onChangeRef);
  }, [containerRef, lastValueRef, extensions, onChangeRef]);

  _react["default"].useLayoutEffect(function () {
    var _containerRef$current;

    if (!editor) {
      return;
    }

    (_containerRef$current = containerRef.current) === null || _containerRef$current === void 0 ? void 0 : _containerRef$current.append(editor.dom);
    return function () {
      editor === null || editor === void 0 ? void 0 : editor.destroy();
    };
  }, [containerRef, editor]);

  effects = ((_effects = effects) === null || _effects === void 0 ? void 0 : _effects.length) === 0 ? undefined : effects;

  _react["default"].useLayoutEffect(function () {
    if (effects && effects.length > 0 && !editor) {
      console.error('received effects but editor is not ready', effects);
      return;
    }

    if (!effects || !editor) return;
    effects.forEach(function (effect) {
      effect(editor);
    });
  }, [effects, editor]);

  _react["default"].useEffect(function () {
    if (!editor) {
      return;
    }

    editor.dispatch({
      effects: readOnlyCompartment.current.reconfigure(_view.EditorView.editable.of(Boolean(onChange)))
    });
  }, [editor, onChange]);

  _react["default"].useLayoutEffect(function () {
    if (editor && containerRef.current && lastValueRef.current !== value) {
      if (lastValueRef.current === value) {
        return;
      }

      var textA = editor.state.doc.toString();
      var textB = value;
      onExternalChange(editor, textB, textA);
      lastValueRef.current = textB;
    } else {
      lastValueRef.current = value;
    }
  }, [containerRef, value, editor, onExternalChange]);

  return editor;
};

exports.useCodeMirror = useCodeMirror;